//C++ function to read jpeg coefficients and quantization table from a jpeg file
//input: Filename
//ouput: [coefficients,quantization table]
//Only quadratic grayscale images are allowed, whose size is a multiple of 8

#include "mex.h"

#include <iostream>
#include <jpeglib.h>
#include <stdio.h>
//#include <math.h>

using namespace std;

void mexFunction(int nlhs, mxArray *plhs[], 
    int nrhs, const mxArray *prhs[])
{

   //Get image data-----------------------------------------------------------------------
   //-------------------------------------------------------------------------------------
   char *infilename; //Char to store image name
   int buflen,status;
   
   /* Get the length of the input string. */
   buflen = (mxGetM(prhs[0]) * mxGetN(prhs[0])) + 1;

   infilename = (char *) mxCalloc(buflen, sizeof(char)); //Allocate memory for image name
   
   status = mxGetString(prhs[0], infilename, buflen);  //Set infilename to image name
   
   
   //Initialize JPEG library for data reading--------------------------------
   
     //Note that under comment also jpeg compression is prepared
   	//Initialize infile and outfile
   FILE *infile;//,*outfile;
  
   
   	//Open infile and outfile
   infile = fopen(infilename, "rb");
   //outfile= fopen(outfilename,"w");


	//Allocate decompression and compression data
   jpeg_decompress_struct srcinfo;
   //jpeg_compress_struct dstinfo;
   jpeg_error_mgr jsrcerr;//, jdsterr;

   //Initialize decompression object
   srcinfo.err = jpeg_std_error(&jsrcerr);
   jpeg_create_decompress(&srcinfo);
   
	
	
	//Specify data source for decompression
   jpeg_stdio_src(&srcinfo, infile);
   
        //Read source header
   jpeg_read_header(&srcinfo, TRUE);
   
   	//Set image dimensions
   int N=srcinfo.image_height;
   int M=srcinfo.image_width;
 
 	//Copy critical parameters to destination
   //jpeg_copy_critical_parameters(&srcinfo, &dstinfo);
   
   //Get quanitzation table--------------------------------------------------------

   //Allocate UINT16 array for the quantization table
   UINT16 *table;
   table = new UINT16[64];
   
   //Get table structure from srcinfo
   table=srcinfo.quant_tbl_ptrs[0][0].quantval;
   
   jpeg_component_info componinfo = *srcinfo.comp_info;
   
   int bh=componinfo.height_in_blocks;
   int bw=componinfo.width_in_blocks;
   

   //Get the DCT coefficients-------------------------------------------------------
   
   
   	//Allocate virtual block array pointer
   jvirt_barray_ptr *v_barray;
   
   	//Get virtual array pointer
   v_barray =  jpeg_read_coefficients(&srcinfo);
 
   	//Get block array, barray[row_nr][block_nr][coeff_nr]
   JBLOCKARRAY barray=srcinfo.mem[0].access_virt_barray((j_common_ptr) &srcinfo,v_barray[0],(JDIMENSION) 0,(JDIMENSION) 1,FALSE);
   

   //End of getting image data---------------------------------------------------------------------------------
   //Given: N,M=image size, table[element number], barray[row_nr][block_nr][coeff_nr], bh=N/8,bw=M/8-------------------

   double *coeff_data, *table_data;
 
    // Create an mxArray for the output data
    plhs[0] = mxCreateDoubleMatrix(M, N, mxREAL); //Image data
    plhs[1] = mxCreateDoubleMatrix(8, 8, mxREAL); //Talble data


    // Create a pointer to the output data */
    coeff_data = mxGetPr(plhs[0]); //set output to coeff_data
    table_data = mxGetPr(plhs[1]); //set output to table_data


     //Write coefficient output, note that output is transposed
 
   for (int row=0;row<bh;++row)
   {
    for (int bl_nr=0;bl_nr<bw;++bl_nr)
    {
     for (int coef_nra=0;coef_nra<8;++coef_nra)
     {
      for (int coef_nrb=0;coef_nrb<8;++coef_nrb)
      {
       coeff_data[row*bw*64 + 8*bl_nr + M*coef_nra + coef_nrb]=barray[row][bl_nr][coef_nra*8+coef_nrb];
      }
     }
    }
   }

    //Write table output, note that output is transposed
    for (int i=0;i<64;++i)
    {
      table_data[i]=table[i]; //Write in colomn-major format
    }
    
    
    //Deallocate Data
    

    	//Finish JPEG decompression object
   jpeg_finish_decompress(&srcinfo);
   jpeg_destroy_decompress(&srcinfo);  
   
   	//Close files
   fclose(infile);
   
}

