%Authors: 	Martin Holler and Kristian Bredies
%		University of Graz
%Contact	martin.holler@uni-graz.at

function [g1,g2,tv,gap] = tv(image_name,tolerance,mode)

%Preparative###########################################################################################################

%Set pathh
addpath standard_functions

%Read image
g = imread(image_name);
g = double(g);

%Set parameter

sig=sqrt(1/8);
tau=sqrt(1/8);


%Encode
if strcmp(mode{1},'simulate')
	rate = mode{2};
	[data,A] = jpeg_enc(g,rate); %Simulate JPEG data
elseif strcmp(mode{1},'realdata')
	[data,A]=read_data(image_name); %Read coefficients and quantization table from file
	data=double(data');
	A=double(A');
end


%Set standard decompression
g1 = jpeg_dec(data,A);
g1 = uint8(g1);

%Get image size
[n,m] = size(g1);

%Set data boundaries
ldata = zeros(n,m);
rdata = zeros(n,m);
for n_it=1:8:n
	for m_it = 1:8:m
		ldata(n_it:n_it+7,m_it:m_it+7) = (  data(n_it:n_it+7,m_it:m_it+7) -0.5).*A;
		rdata(n_it:n_it+7,m_it:m_it+7) = (  data(n_it:n_it+7,m_it:m_it+7) +0.5).*A;
		%Dequantize data
		data(n_it:n_it+7,m_it:m_it+7) = data(n_it:n_it+7,m_it:m_it+7).*A;
	end
end

%Initialize standard variable
k = 0;

%%Algorithmic######################################################################################################


		%Initialize regularization dependent variable
		x = zeros(n,m);
		x = double(g1);
		ext = x;
		y = zeros(n,m,2);
		
		%Total variation and primal-dual gap
		tv = zeros(1,500);
		gap = zeros(1,500);
		
		tv(1) = get_tv(x);
		gap(1) = tv(1) + get_gstar(y,data,A);


%Decompression loop
while gap(k+1)/(n*m) > tolerance
    
    %Dual
    
    	%y = y + sig*K(ext)
    	y = y + sig*fgrad_1( ext(:,:,1) );
        
        %Projection
	n1 = frob_norm_2( y );
	n1 = max(1,n1);
	for i=1:2
		y(:,:,i) = y(:,:,i)./n1;
	end

    %Primal
    
	%Set x+=x-tau*A'(z)
    	ext=x-tau*( -bdiv_1( y(:,:,1:2) ) );
    	
    	%Project to data
    	ext = project_to_data( ext, ldata, rdata );
    	
    %Extragradient
    
    	%Set extragradient
    	x=2*ext - x;
    
	%Swap variables
    	[x,ext] = deal(ext,x);
        

    k = k+1;

    if (rem(k,10) == 0)
    	display(['Iteration: ',num2str(k)]);
    end
    
    %Enlarge
    if (rem(k,500) == 0)
    	tv = [tv,zeros(1,500)];
    	gap = [gap,zeros(1,500)];
    end
    
    %Get tv and gap
    tv(k+1) = get_tv(x);
    gap(k+1) = tv(k+1) + get_gstar(y,data,A);
    
end 


%Crop
tv = tv(1,1:k+1);
gap = gap(1,1:k+1);

%Finalizing#########################################################################################################

%Set result
g2 = x(:,:,1) + 128;

g2 = uint8(g2);




